package host;

//import java.util.concurrent.LinkedBlockingQueue;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.concurrent.ConcurrentLinkedQueue;

/**
 * DataAnalyzer class, handles all data analysis
 * @author bentzx
 *
 */
public class DataAnalyzer {

	private Panel panel_;
	private SerialManager serialManager_;
//	private LinkedBlockingQueue<Integer> queue_= null;
	private int preambleByteCount_ = 0;
	public static final int BMP_SIZE = 48694;
//	private int testCount=0;
	private ConcurrentLinkedQueue<Integer> queue_ = null;
	private int trackLatch_ = 0;

	
	/**
	 * Constructor
	 * @param p Panel
	 * @param sM SerialManager
	 */
	public DataAnalyzer(Panel p, SerialManager sM){
		panel_ = p;
		serialManager_ = sM;
//		queue_ = new LinkedBlockingQueue<Integer>();
		queue_ = new ConcurrentLinkedQueue<Integer>();
	}
	
	/**
	 * Receives serial port data
	 * @param i 
	 */
	public void rxData(int i){
		detectPreamble(i);
		System.out.println(preambleByteCount_);
		if (preambleByteCount_>=4 && preambleByteCount_ <=8){

			queue_.add(new Integer(i));
		}
		if (preambleByteCount_ == 8){
			System.out.println("qS: " + queue_.size());
			if (queue_.size()==BMP_SIZE+5){
				createPic();
			}
//			serialManager_.autoReconnect();
			preambleByteCount_ = 0;
			queue_.clear();
			System.out.println("qS: " + queue_.size());
		}
	}
	
	/**
	 * Converts int to hex
	 * @param i int
	 * @return hex
	 */
	private String intToHex(int i) {
		//String hex = Integer.toHexString(i);
		String hex = Integer.toHexString(0x100 | (0x0ff & i)).substring(1);
		if (hex.length() == 1) {
			hex = "0" + hex;
		}
		return hex;
	}
	
	/**
	 * Creates bitmap file from image data
	 */
	private void createPic() {
		FileOutputStream fos;
		File tempFile = new File("recordings/temp.bmp");
		try {
			fos = new FileOutputStream (tempFile);
			queue_.poll(); //remove the first element since not needed and unnecessarily added - the last preamble byte
			Integer [] queueArray = queue_.toArray(new Integer[0]);
			for (int i=0; i<queueArray.length-4; i++){ //ignore the subsequent preamble stored
				fos.write(queueArray[i].intValue());
			}
			panel_.initDisplay(tempFile);
			fos.close();
			fos = null;
			System.out.println("initDisplay carried out");
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		preambleByteCount_ = 0;
		queue_.clear();
	}
	
	/**
	 * To check for preamble, start incrementing preambleByteCount everytime int i matches preamble sequence.
	 * Keep rxData running as long as preambleByteCount is 4, 5, 6, 7; when reaches 8, know that reached end of second preamble,
	 * then createPic and be sure to deduct the subsequent preamble from queue first
	 * @param i integer data received
	 * 
	 */
	private void detectPreamble(int i) {
		if (intToHex(i).equals("00") && preambleByteCount_==0){
			preambleByteCount_++;
		} else if (intToHex(i).equals("00") && preambleByteCount_==1){
			preambleByteCount_++;
		} else if (intToHex(i).equals("be") && preambleByteCount_==2){
			preambleByteCount_++;
		} else if (intToHex(i).equals("36") && preambleByteCount_==3){
			preambleByteCount_++;
			System.out.println("Found preamble");
			panel_.getStatusTF().setText("Capturing image...");
		} else if (intToHex(i).equals("aa") && preambleByteCount_==4){
			preambleByteCount_=5;
		} else if (intToHex(i).equals("aa") && preambleByteCount_==0 && (trackLatch_>30000)){	//get system out of latch
			preambleByteCount_=5;
			serialManager_.autoReconnect();
//			trackLatch_ = 0;	//reset trackLatch
			panel_.getStatusTF().setText("Auto reconnect...");
		} else if (intToHex(i).equals("55") && preambleByteCount_==5){
			preambleByteCount_=6;
		} else if (intToHex(i).equals("a5") && preambleByteCount_==6){
			preambleByteCount_=7;
		} else if (intToHex(i).equals("5a") && preambleByteCount_==7){
			preambleByteCount_=8;
			trackLatch_ = 0;	//reset trackLatch
			System.out.println("Found subsequent preamble");
//		} else if (intToHex(i).equals("00") && preambleByteCount_==8){
//			preambleByteCount_ = 1;
		} else {
			if (preambleByteCount_<=3 || preambleByteCount_ ==8){
				preambleByteCount_ = 0;
				if (preambleByteCount_==0){
					trackLatch_++;
				}
				
			} else {
				preambleByteCount_ = 4;
			}
		}
	}
}
 
