package host;

import java.awt.event.ActionEvent;
import java.io.IOException;

import javax.comm.PortInUseException;
import javax.comm.UnsupportedCommOperationException;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

/**
 * HostManager class
 * Main
 * @author bentzx
 */
public class HostManager extends JFrame{

	private static final long serialVersionUID = 1L;
	private Panel panel_; 					//Panel
	private DataAnalyzer dataAnalyzer_; 		//DataAnaylzer
	private SerialManager serialManager_;	//SerialManager
	
	private JMenuItem helpMenuAbout_;
    private JMenuItem fileMenuExit_;
    private JMenu fileMenu_;
    private JMenu helpMenu_;
    private JMenuBar menuBar_;
	
	public static final String PROJECT_NAME = "Wireless Video Camera";
	
	/**
	 * HostManager constructor
	 * @throws IOException
	 * @throws UnsupportedCommOperationException
	 * @throws PortInUseException
	 */
	public HostManager() throws IOException, UnsupportedCommOperationException, PortInUseException{
		super();
		//create in correct sequence
		serialManager_ = new SerialManager(this);
		panel_ = new Panel(serialManager_);
		dataAnalyzer_ = new DataAnalyzer(panel_, serialManager_);
		initComponents();
//		panel_.testRecorder();
	}
	
	/**
	 * Initialize components on frame
	 */
	private void initComponents(){
		setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setBackground(new java.awt.Color(255, 255, 255));
        setResizable(false);
        setTitle(HostManager.PROJECT_NAME);

        getContentPane().add(panel_, java.awt.BorderLayout.CENTER);
        initMenu();
        initListeners();

        pack();
        setLocation(250, 250);

        setVisible(true);
	}
	
	/**
	 * Initialize menu bar items
	 */
	private void initMenu(){
		menuBar_ = new javax.swing.JMenuBar();
        fileMenu_ = new javax.swing.JMenu();
        fileMenuExit_ = new javax.swing.JMenuItem();
        helpMenu_ = new javax.swing.JMenu();
        helpMenuAbout_ = new javax.swing.JMenuItem();
		
		fileMenu_.setText("File");
		fileMenuExit_.setText("Exit");
		fileMenu_.add(fileMenuExit_);
		menuBar_.add(fileMenu_);
		helpMenu_.setText("Help");
		helpMenuAbout_.setText("About");
		helpMenu_.add(helpMenuAbout_);
		menuBar_.add(helpMenu_);
		setJMenuBar(menuBar_);
	}
	
	 /**
     * Initialize all listeners.
     */
    private void initListeners() {
    	//add action listener to exit menu item
        fileMenuExit_.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fileMenuExitActionPerformed(evt);
            }
        });
        
      //add action listener to about menu item
        helpMenuAbout_.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                helpMenuAboutActionPerformed(evt);
            }
        });
    }
	
    /**
     * Handle action event concerning "exit" menu item.
     * @param evt The ActionEvent
     */
    private void fileMenuExitActionPerformed(ActionEvent evt) {
        System.exit(0);
    }
    
    /**
     * Handle action event concerning "about" menu item.
     * @param evt The ActionEvent
     */
    private void helpMenuAboutActionPerformed(ActionEvent evt) {
        String[] msg = {HostManager.PROJECT_NAME + " Ver 1.0"};
        JOptionPane.showMessageDialog(this, msg, "About",
                                      JOptionPane.INFORMATION_MESSAGE);
    }
	
	/**
	 * To get DataAnalyzer instance
	 * @return DataAnalyzer instance
	 */
	public DataAnalyzer getDataAnalyzer() {
		return dataAnalyzer_;
	}
    
    
	/**
     * Main method called at start of program.
     * @param argv Command line argument
	 * @throws IOException 
	 * @throws PortInUseException 
	 * @throws UnsupportedCommOperationException 
     */
	public static void main(String [] argv) throws IOException, UnsupportedCommOperationException, PortInUseException {
		new HostManager();
	}
}
